#include "cuda.h"

#define LEN 100000

// GPU code
__global__ void square(float *data, int length)
{
  int index = blockIdx.x*blockDim.x + threadIdx.x;
  if (index<length)
  {
     data[index]=data[index]*data[index];
  }
}

//Host code
int main()
{
  float * host_data, *gpu_data;
  int ThreadsPerBlock, Blocks;

  // Allocate memory
  host_data=(float*)malloc(LEN*sizeof(int));
  cudaMalloc(&gpu_data,LEN*sizeof(int));

  // Initialize data on host
  for(int i=0; i<LEN; i++)
  {
    host_data[i]=2*i;  
  }

  // Copy host data to GPU
  cudaMemcpy(gpu_data,host_data,LEN*sizeof(int),cudaMemcpyHostToDevice);

  // Perform computation on GPU
  ThreadsPerBlock=128;
  Blocks = (int)((LEN-1)/ThreadsPerBlock) + 1;
  square<<<Blocks, ThreadsPerBlock>>>(gpu_data,LEN);

  // Copy GPU data back to host
  cudaMemcpy(gpu_data,host_data,LEN*sizeof(int),cudaMemcpyDeviceToHost);

  // Free allocated memory
  cudaFree(gpu_data);
  free(host_data);
}
